/* FL/abi-version.h.  Generated from abi-version.in by configure.  */
/*
  ABI Configuration file for the Fast Light Tool Kit (FLTK).

  ============================================================================
    DO NOT EDIT - This file is generated by configure !
  ============================================================================

  define FL_ABI_VERSION: 1xxyy for 1.x.y (xx,yy with leading zero)
*/

/* #undef FL_ABI_VERSION */
